from typing import Any, Dict, List, Optional
from ogd.core.generators.Generator import GeneratorParameters
from ogd.core.generators.extractors.Feature import Feature
from ogd.core.models.Event import Event
from ogd.core.models.enums.ExtractionMode import ExtractionMode
from ogd.core.models.FeatureData import FeatureData

class PersistThroughFailure(Feature):
    def __init__(self, params: GeneratorParameters):
        super().__init__(params=params)
        self.failed: bool = False
        self.persisted: int = 0

    # Implement abstract functions
    @classmethod
    def _eventFilter(cls, mode: ExtractionMode) -> List[str]:
        # List of player action events + lose_game for failure tracking
        return [
            "game_start", "select_policy_card", "click_build", 
            "click_destroy", "click_undo", "click_execute_build", 
            "click_confirm_destroy", "lose_game"
        ]

    @classmethod
    def _featureFilter(cls, mode: ExtractionMode) -> List[str]:
        return []

    def _updateFromEvent(self, event: Event) -> None:
        event_type = event.EventName
        if event_type == "lose_game":
            self.failed = True
        elif self.failed and event_type in [
            "game_start", "select_policy_card", "click_build", 
            "click_destroy", "click_undo", "click_execute_build", 
            "click_confirm_destroy"
        ]:
            self.persisted += 1
            self.failed = False

    def _updateFromFeatureData(self, feature: FeatureData):
        return

    def _getFeatureValues(self) -> List[Any]:
        return [self.persisted > 0]

    # Subfeature "count"
    def _getSubfeatureValues(self) -> Dict[str, Any]:
        return {"count": self.persisted}

    # Optionally override public functions
    @staticmethod
    def MinVersion() -> Optional[str]:
        return "1"



from typing import Any, Dict, List, Optional
from ogd.core.generators.Generator import GeneratorParameters
from ogd.core.models.Event import Event
from ogd.core.models.enums.ExtractionMode import ExtractionMode
from ogd.core.models.FeatureData import FeatureData
from ogd.games.BLOOM.features.PerCountyFeature import PerCountyFeature

class CountyFailCount(PerCountyFeature):
    def __init__(self, params: GeneratorParameters):
        super().__init__(params=params)
        self.county_fail_counts: Dict[str, Dict[str, int]] = {}

    # *** IMPLEMENT ABSTRACT FUNCTIONS ***
    @classmethod
    def _eventFilter(cls, mode: ExtractionMode) -> List[str]:
        return ["lose_game"]

    @classmethod
    def _featureFilter(cls, mode: ExtractionMode) -> List[str]:
        return []

    def _updateFromEvent(self, event: Event) -> None:
        county_name = event.EventData.get("county_name", None)
        fail_type = event.EventData.get("lose_condition", "")

        if county_name:
            if county_name not in self.county_fail_counts:
                self.county_fail_counts[county_name] = {
                    "Total": 0,
                    "CityFailed": 0,
                    "TooManyBlooms": 0,
                    "OutOfMoney": 0
                }

            self.county_fail_counts[county_name]["Total"] += 1

            if fail_type in self.county_fail_counts[county_name]:
                self.county_fail_counts[county_name][fail_type] += 1

    def _updateFromFeatureData(self, feature: FeatureData):
        return

    def _getFeatureValues(self) -> List[Any]:
        all_values = []
        for county_name, fail_counts in self.county_fail_counts.items():
            all_values.extend([
                fail_counts["Total"],
                fail_counts["CityFailed"],
                fail_counts["TooManyBlooms"],
                fail_counts["OutOfMoney"]
            ])
        return all_values

    def Subfeatures(self) -> List[str]:
        return ["CityFailed", "TooManyBlooms", "OutOfMoney"]

    # *** Optionally override public functions. ***
    @staticmethod
    def MinVersion() -> Optional[str]:
        return "1"



from typing import Any, Dict, List, Optional
from ogd.core.generators.Generator import GeneratorParameters
from ogd.core.generators.extractors.Feature import Feature
from ogd.core.models.Event import Event
from ogd.core.models.enums.ExtractionMode import ExtractionMode
from ogd.core.models.FeatureData import FeatureData

class AlertCount(Feature):
    def __init__(self, params: GeneratorParameters):
        super().__init__(params=params)
        self.alert_counts: Dict[str, int] = {
            "DieOff": 0,
            "DecliningPop": 0,
            "SellingLoss": 0,
            "CritImbalance": 0
        }

    # *** IMPLEMENT ABSTRACT FUNCTIONS ***
    @classmethod
    def _eventFilter(cls, mode: ExtractionMode) -> List[str]:
        return ["local_alert_displayed"]

    @classmethod
    def _featureFilter(cls, mode: ExtractionMode) -> List[str]:
        return []

    def _updateFromEvent(self, event: Event) -> None:
        alert_type = event.EventData.get("alert_type", "")
        if alert_type in self.alert_counts:
            self.alert_counts[alert_type] += 1

    def _updateFromFeatureData(self, feature: FeatureData):
        return

    def _getFeatureValues(self) -> List[Any]:
        return [
            sum(self.alert_counts.values()),
            self.alert_counts["DieOff"],
            self.alert_counts["DecliningPop"],
            self.alert_counts["SellingLoss"],
            self.alert_counts["CritImbalance"]
        ]

    def Subfeatures(self) -> List[str]:
        return ["DieOff", "DecliningPop", "SellingLoss", "CritImbalance"]

    # *** Optionally override public functions. ***
    @staticmethod
    def MinVersion() -> Optional[str]:
        return "1"



# import libraries
import logging
from typing import Any, Dict, List, Optional
# import locals
from ogd.core.utils.Logger import Logger
from ogd.core.generators.Generator import GeneratorParameters
from ogd.core.models.Event import Event
from ogd.core.models.enums.ExtractionMode import ExtractionMode
from ogd.core.models.FeatureData import FeatureData
from ogd.core.generators.extractors.SessionFeature import SessionFeature


class LogVersion(SessionFeature):

    def __init__(self, params:GeneratorParameters):
        super().__init__(params=params)
        self._current_log : str = "UNKNOWN LOG VERSION"

    # *** IMPLEMENT ABSTRACT FUNCTIONS ***
    @classmethod
    def _eventFilter(cls, mode:ExtractionMode) -> List[str]:
        return ["all_events"]

    @classmethod
    def _featureFilter(cls, mode:ExtractionMode) -> List[str]:
        return []

    def _updateFromEvent(self, event:Event) -> None:
        self._current_log = event.LogVersion


    def _updateFromFeatureData(self, feature:FeatureData):
        return

    def _getFeatureValues(self) -> List[Any]:
        return [self._current_log]
    
    # *** Optionally override public functions. ***

    @staticmethod
    def AvailableModes() -> List[ExtractionMode]:
        return [ExtractionMode.SESSION]

    